#pragma once

#ifndef ShutterStream_TurntableHardware
#define ShutterStream_TurntableHardware

#include "SSAbstractPlugin.h"

class SHUTTERSTREAM_API SSTurntableHardware : public SSAbstractPlugin
{
public:
    typedef float* CustomAnglesArray;

    ~SSTurntableHardware();

    SSRegisterPlugin::PluginType pluginType() const override final;

    //! override it for host to know how many turntables are available/found from/in this plug-in
    /*!
     \return turntables names list
     \sa setCurrentTurntableIndex
     */
    virtual DevicesNamesList availableDevices() const = 0;

    //! override it for host to open a turntable device
    /*
    \param the index of turntable to open
    \sa availableDevices
    \note it's up to the plug-in how to implement it, async or sync, the plug-in must signal the host when turntables was opened or failed
    \sa hardwareOpenSucceeded
    \sa hardwareDisconnected
    */
    virtual void openDevice(DevicesNamesList::size_type index) = 0;

    //! override it for host to close the current turntable device index
    /*
    \sa currentTurntableIndex
    */
    virtual void closeDevice() = 0;

    //! override it if additional settings are required
    /*
     \param imagesCount must be filled with complete 360 product photography images count
     \param customAngles if not null and supported by the plug-in   - must be allocated with "new [imagesCount]", and filled with custom angles
                                                                    - the caller will delete the array
     \return true if all settings are ok
     \note *must* call the base class with imagesCount filled, which internally set the complete 360 product photography images count
     \note it is always called before the pluginUIFrameViewWillClose, can load the settings from UI
     */
    virtual bool complete360ProductPhotographyWillStart(uint16_t* imagesCount, CustomAnglesArray* customAngles = nullptr);

    //! override it for host to rotate the turntable to a specific angle
    /*
    \param angle rotation's angle pos
        - positive number means clockwise rotation
        - negative means counterclockwise rotation
    \note the plug-in must signal host when the turntable is stopped from rotation (even if the device cannot start the rotation)
    \note *must first* call the base class function, which increase 360 product photography image index (if 360 product photography is started)
    \note angle can be converted to steps and vice-versa
    \sa stepsToDegrees
    \sa degreesToSteps
    \sa turntableRotationStopped
    \sa complete360ProductPhotographyImageIndex
    \return true is rotating is stared successfully
    */
    virtual bool rotateDeviceToAngle(float anglePos);

    //! override it for host to stops the device from spinning
    /*
    \note *must* call the base class function for resetting the image index
     */
    virtual void stopsSpinning();

    //! override it for host to creating the turntable controls Ui
    virtual SSUIFrameView createTurntableUIFrameView();


    // image index for current 360 product photography
    /*
     \return image index for current 360 product photography
     \sa rotateDeviceWithSteps
     */
    uint16_t complete360ProductPhotographyImageIndex() const;

    //! images count for current 360 product photography
    /*
     \param customAngles if supported is filled with custom angles
     \return images count for current 360 product photography
     */
    uint16_t complete360ProductPhotographyImagesCount(CustomAnglesArray* customAngles) const;

    //! steps to degrees
    /*
     \param steps number of steps to be converted into degrees
        - positive -> clockwise
        - negative <- counterclockwise
     \return number of degrees for coressponding steps
     \sa complete360RotationStepsCount
     */
    float stepsToDegrees(short steps) const;

    //! degrees to steps
    /*
     \param degrees number of degrees to be converted into steps
        - positive -> clockwise
        - negative <- counterclockwise
     \return number of steps for corresponding degrees
     */
    short degreesToSteps(float degrees) const;

    //! called by the host to set the camera index
    void setCurrentTurntableIndex(DevicesNamesList::size_type index);
    //! called by the plug-in to retrieve the current camera index
    /*
    \return current turntable index if any, else std::numeric_limits<DevicesNamesList::size_type>::max()
     */
    DevicesNamesList::size_type currentTurntableIndex() const;


protected:
    SSTurntableHardware();

    //! all the below functions are thread-safe
    SS_THREAD_SAFEIMPL

        //! signal the host the turntable is opened
        /*!
        \param index the turntable index
        \param complete360RotationStepsCount steps count for a complete 360 rotation
        \param canMoveCounterclockwise true if turntable support counterclockwise rotation
        \note host will call setCurrentTurntableIndex in this function
        \sa setCurrentTurntableIndex
        */
        SS_SIGNAL void hardwareOpenSucceeded(DevicesNamesList::size_type index, uint32_t complete360RotationStepsCount, bool canMoveCounterclockwise);

    //! signal the host the turntable is disconnected
    /*!
    \param index the disconnected turntable's index
    */
    SS_SIGNAL void hardwareDisconnected(DevicesNamesList::size_type index);

    //! signal the host the turntable has stoppped from the rotation
    SS_SIGNAL void turntableRotationStopped();


    DECLARE_SSPIMPL(TurntableHardware)
};

#endif
